import unittest
from datetime import datetime
from unittest.mock import patch

import requests_mock
from pytz import UTC

from ._utils.mock_classes import MockApiClient, MockRefreshTokenApiClient, MockAccessTokenApiClient
from ._utils.t_case_mixins import InstantiateMixin, ApiClientSharedTestsMixin


class MustBeGottenFromCache(Exception):
    pass


class TestBaseApiClient(InstantiateMixin, ApiClientSharedTestsMixin, unittest.TestCase):
    def setUp(self):
        self.api_client = MockApiClient()

    def test_request_other_errors(self):
        with requests_mock.mock() as m:
            with self.assertRaises(AssertionError):
                self.api_client.request(self.api_client.GET, self.path1, base=self.url1, sec_id_for_special_token='foo')


class TestSecRefreshTokenMixin(InstantiateMixin, ApiClientSharedTestsMixin, unittest.TestCase):
    def setUp(self):
        self.api_client = MockRefreshTokenApiClient()

    def test_get_sec_refresh_token(self):
        with requests_mock.mock() as m:
            m.get(self.url_refresh_path_refresh, text=self.fixtures[self.FIXTURE_REFRESH_TOKEN], status_code=200)
            refresh_token = self.api_client.get_sec_refresh_token('123')
            self.assertEqual('bar', refresh_token)
            self.assertEqual('bar', self.api_client._get_cache_sec_refresh_token('123'), 'Refresh token must be cached')
        # Now refresh token must be gotten from cache
        with requests_mock.mock() as m:
            m.get(self.url_refresh_path_refresh, exc=MustBeGottenFromCache)
            refresh_token = self.api_client.get_sec_refresh_token('123')
            self.assertEqual('bar', refresh_token)


class RetryEmulationApiClient(MockAccessTokenApiClient):
    def __init__(self, request_mock_callback, *args, **kwargs):
        self.request_mock_callback = request_mock_callback
        super(RetryEmulationApiClient, self).__init__(*args, **kwargs)

    def _retry_occurred(self):
        """
        For emulation the situation when token was expired while request is processing,
        we need to make a new mock with good response
        """
        self.request_mock_callback()


class TestSecAccessTokenMixin(InstantiateMixin, ApiClientSharedTestsMixin, unittest.TestCase):
    def setUp(self):
        self.api_client = MockAccessTokenApiClient()

    def test_get_public_key(self):
        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            public_key = self.api_client.get_public_key()
            self.assertEqual(fixture_public_key, public_key)
            self.assertEqual(fixture_public_key, self.api_client._get_cache_public_key(), 'Public key must be cached')
        # Now refresh token must be gotten from cache
        with requests_mock.mock() as m:
            m.get(self.url_public_key_path_public_key, exc=MustBeGottenFromCache)
            public_key = self.api_client.get_public_key()
            self.assertEqual(fixture_public_key, public_key)

    def test_parse_access_token_expiration(self):
        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            expiration = self.api_client.parse_access_token_expiration(self.fixtures[self.FIXTURE_ENCODED_TOKEN])
            self.assertEqual(datetime(year=2018, month=3, day=22, hour=10, minute=4, second=5, tzinfo=UTC), expiration)

    def test_is_sec_access_token_expired(self):
        self.assertTrue(self.api_client._is_sec_access_token_expired('123'),
                        'Must be expired because empty cache')

        # for example token expiration is set (do it next line)
        self.api_client.\
            _set_cache_sec_access_token_expiration('123',
                                                   datetime(year=2018, month=3, day=22,
                                                            hour=10, minute=4, second=5, tzinfo=UTC))

        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=3, second=0, tzinfo=UTC)
                self.assertFalse(self.api_client._is_sec_access_token_expired('123'),
                                 'Must not be expired because is about minute difference')

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=4, second=0, tzinfo=UTC)

                self.assertTrue(self.api_client._is_sec_access_token_expired('123'),
                                'Must be expired because is about 5 seconds difference')

    def test_get_sec_access_token(self):
        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            m.post(self.url_oauth_path_oauth_token, text=self.fixtures[self.FIXTURE_ACCESS_TOKEN], status_code=200)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                # (expired)
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=4, second=0, tzinfo=UTC)

                access_token = self.api_client.get_sec_access_token('123')
                self.assertEqual(self.fixtures[self.FIXTURE_ENCODED_TOKEN], access_token)

                cached_expiration = self.api_client._get_cache_sec_access_token_expiration('123')
                self.assertEqual(datetime(year=2018, month=3, day=22, hour=10, minute=4, second=5, tzinfo=UTC),
                                 cached_expiration)

                cached_access_token = self.api_client._get_cache_sec_access_token('123')
                self.assertEqual(self.fixtures[self.FIXTURE_ENCODED_TOKEN], cached_access_token)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                # (not expired)
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=3, second=0, tzinfo=UTC)
                m.post(self.url_oauth_path_oauth_token, exc=MustBeGottenFromCache)
                access_token = self.api_client.get_sec_access_token('123')
                self.assertEqual(self.fixtures[self.FIXTURE_ENCODED_TOKEN], access_token)

    def test_full_cycle_request(self):
        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            m.post(self.url_oauth_path_oauth_token, text=self.fixtures[self.FIXTURE_ACCESS_TOKEN], status_code=200)
            m.get(self.url_using_sec_token_path_using_sec_token,
                  text=self.fixtures[self.FIXTURE_SIMPLE_RESPONSE], status_code=200)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                # (not expired)
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=3, second=0, tzinfo=UTC)

                r = self.api_client.request(self.api_client.GET,
                                            self.path_using_sec_token,
                                            base=self.url_using_sec_token, sec_id_for_special_token='123')

                self.assertEqual('ok', r['status'])

    def test_full_cycle_request_with_retry(self):
        with requests_mock.mock() as m:
            def _callback():
                m.get(self.url_using_sec_token_path_using_sec_token,
                      text=self.fixtures[self.FIXTURE_SIMPLE_RESPONSE], status_code=200)

            # We have to use a special client for test this
            retry_emu_api_client = RetryEmulationApiClient(_callback)

            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            m.post(self.url_oauth_path_oauth_token, text=self.fixtures[self.FIXTURE_ACCESS_TOKEN], status_code=200)
            m.get(self.url_using_sec_token_path_using_sec_token, status_code=401)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                # (not expired)
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=3, second=0, tzinfo=UTC)

                r = retry_emu_api_client.request(self.api_client.GET,
                                                 self.path_using_sec_token,
                                                 base=self.url_using_sec_token, sec_id_for_special_token='123')

                self.assertEqual('ok', r['status'])

    def test_full_cycle_request_with_retry_then_fail(self):
        with requests_mock.mock() as m:
            fixture_public_key = self.fixtures[self.FIXTURE_PUBLIC_KEY]
            m.get(self.url_public_key_path_public_key, text=fixture_public_key, status_code=200)
            m.post(self.url_oauth_path_oauth_token, text=self.fixtures[self.FIXTURE_ACCESS_TOKEN], status_code=200)
            m.get(self.url_using_sec_token_path_using_sec_token, status_code=400)

            # noinspection PyUnresolvedReferences
            with patch.object(MockAccessTokenApiClient, '_get_time_now') as _get_time_now:
                # (not expired)
                _get_time_now.return_value = datetime(year=2018, month=3, day=22, hour=10,
                                                      minute=3, second=0, tzinfo=UTC)

                with self.assertRaises(self.api_client.HttpError):
                    self.api_client.request(self.api_client.GET,
                                            self.path_using_sec_token,
                                            base=self.url_using_sec_token, sec_id_for_special_token='123')



import json
import os
import socket
from datetime import datetime
from typing import Tuple

from pytz import UTC

from shortesttrack_tools.logger.channels import LogChannel
from shortesttrack_tools.logger.clients.base_logger_client import BaseLoggerClient


class PrototypeLoggerClient(BaseLoggerClient):
    def __init__(self, channel: LogChannel, logger_key: str = os.getenv('PROTOTYPE_LOGGER_KEY')):
        self._pre_installed_logger_key = logger_key
        super().__init__()

        self._channel = channel

        try:
            self._socket = socket.socket()
            self._socket.connect(self._connection_address)
        except Exception:
            self._meta_logger.exception('Error on socket connection to logger service')
        else:
            self._is_connected = True

    def send(self, message: str):
        if self._is_connected:
            serialized_message = self._serialize_message(message)
            # Send messages only if logger is connected
            try:
                self._socket.sendall(serialized_message)
            except Exception:
                self._meta_logger.exception('Error on message send to logger service')
                self._is_connected = False

    def _get_connection_address(self) -> Tuple[str, int]:
        ip = os.getenv('PROTOTYPE_LOGGER_IP')
        port = int(os.getenv('PROTOTYPE_LOGGER_PORT'))
        assert all([ip, port])
        return ip, port

    def _get_key(self):
        return self._pre_installed_logger_key

    def __del__(self):
        self._socket.close()

    def _serialize_message(self, message: str) -> bytes:
        json_message = json.dumps({
            'log_id': self._key,
            'ts': str(datetime.now().astimezone(UTC).isoformat()),
            'ch': self._channel,
            'msg': message,
        })

        json_message += '\n'

        return json_message.encode()



import abc

import six


@six.add_metaclass(abc.ABCMeta)
class BaseLoggerClient:
    def __init__(self, *args, **kwargs):
        from shortesttrack_tools.logger import getLogger
        self._meta_logger = getLogger('prototype-logger-client.meta-logger')
        self._is_connected = False
        try:
            self._connection_address = self._get_connection_address()
            self._key = self._get_key()
        except Exception:
            self._meta_logger.exception('Error on getting logger parameters')

    @abc.abstractmethod
    def send(self, message):
        pass

    @abc.abstractmethod
    def _get_connection_address(self):
        pass

    @abc.abstractmethod
    def _get_key(self):
        pass



import os

import requests
import requests_mock
from urlobject import URLObject

from shortesttrack_tools.api_client.endpoints.endpoint_urls import EndpointURLs

endpoint_urls = EndpointURLs.init()


class InstantiateMixin(object):
    FIXTURE_SIMPLE_RESPONSE = 'simple_response.json'
    FIXTURE_REFRESH_TOKEN = 'refresh_token.json'
    FIXTURE_PUBLIC_KEY = 'fake_public_key'
    FIXTURE_ENCODED_TOKEN = 'fake_encoded_token'
    FIXTURE_ACCESS_TOKEN = 'access_token.json'

    @classmethod
    def setUpClass(cls):
        super(InstantiateMixin, cls).setUpClass()
        cls._dirname = os.path.dirname(os.path.dirname(__file__))
        fixture_names = [
            cls.FIXTURE_SIMPLE_RESPONSE,
            cls.FIXTURE_REFRESH_TOKEN,
            cls.FIXTURE_PUBLIC_KEY,
            cls.FIXTURE_ENCODED_TOKEN,
            cls.FIXTURE_ACCESS_TOKEN
        ]
        cls.fixtures = {name: cls._get_fixture(name) for name in fixture_names}

        cls.url1 = URLObject('http://example.com')
        cls.path1 = 'simple'
        cls.url1_path1 = cls.url1.add_path(cls.path1)

        cls.url_refresh = URLObject(endpoint_urls.UAA_SERVICE_ENDPOINT)
        cls.path_refresh = 'api-keys/123'
        cls.url_refresh_path_refresh = cls.url_refresh.add_path(cls.path_refresh)

        cls.url_public_key = URLObject(endpoint_urls.JWTKEY_ENDPOINT)
        cls.path_public_key = 'token_publickey'
        cls.url_public_key_path_public_key = cls.url_public_key.add_path(cls.path_public_key)

        cls.url_oauth = URLObject(endpoint_urls.OAUTH_SERVICE_ENDPOINT)
        cls.path_oauth_token = 'token'
        cls.url_oauth_path_oauth_token = cls.url_oauth.add_path(cls.path_oauth_token)

        cls.url_using_sec_token = URLObject('http://example.com')
        cls.path_using_sec_token = 'sec'
        cls.url_using_sec_token_path_using_sec_token = cls.url_using_sec_token.add_path(cls.path_using_sec_token)

    @classmethod
    def _get_fixture(cls, fixture_name):
        path = os.path.join(cls._dirname, '_fixtures', fixture_name)
        with open(path) as f:
            return f.read()


class ApiClientSharedTestsMixin(object):
    """
    This is a test for core functionality which shouldn't be broken by mixins
    """
    def test_request_simple(self):
        with requests_mock.mock() as m:
            m.get(self.url1_path1, text=self.fixtures[self.FIXTURE_SIMPLE_RESPONSE], status_code=200)
            r = self.api_client.request(self.api_client.GET, self.path1, base=self.url1)
            self.assertEqual('ok', r['status'])

    def test_request_simple_error(self):
        with requests_mock.mock() as m:
            m.get(self.url1_path1, text=self.fixtures[self.FIXTURE_SIMPLE_RESPONSE], status_code=400)
            with self.assertRaises(self.api_client.HttpError) as context:
                self.api_client.request(self.api_client.GET, self.path1, base=self.url1)

    def test_request_performing_error(self):
        with requests_mock.mock() as m:
            m.get(self.url1_path1,
                  exc=requests.exceptions.RequestException)

            with self.assertRaises(self.api_client.RequestError):
                self.api_client.request(self.api_client.GET, self.path1, base=self.url1)



import os
from urlobject import URLObject

from shortesttrack_tools.unique import Unique


class EndpointURLs(Unique):
    HOST = None

    UAA_SERVICE_ENDPOINT = None
    OAUTH_SERVICE_ENDPOINT = None
    JWTKEY_ENDPOINT = None
    METADATA_SERVICE_ENDPOINT = None
    EXEC_API_SERVICE_ENDPOINT = None
    EXEC_SCHEDULER_SERVICE_ENDPOINT = None
    DATA_SERVICE_ENDPOINT = None
    LOGGING_ENDPOINT = None

    @classmethod
    def _do_init(cls, host: str = None):
        cls.HOST = URLObject(host) if host else URLObject(os.environ.get('HOST', 'https://shortesttrack.com'))

        cls.OAUTH_SERVICE_ENDPOINT = URLObject(os.environ.get('OAUTH_SERVICE_ENDPOINT')) \
            if os.environ.get('OAUTH_SERVICE_ENDPOINT') else cls.HOST.add_path('oauth')

        cls.JWTKEY_ENDPOINT = URLObject(os.environ.get('JWTKEY_ENDPOINT')) \
            if os.environ.get('JWTKEY_ENDPOINT') else cls.HOST.add_path('oauth_jwtkey')

        cls.UAA_SERVICE_ENDPOINT = cls.HOST.add_path('api/uaa')
        cls.METADATA_SERVICE_ENDPOINT = cls.HOST.add_path('api/metadata')
        cls.EXEC_API_SERVICE_ENDPOINT = cls.HOST.add_path('api/execution-metadata/v2')
        cls.EXEC_SCHEDULER_SERVICE_ENDPOINT = cls.HOST.add_path('api/exec-scheduling')
        cls.DATA_SERVICE_ENDPOINT = cls.HOST.add_path('api/data')
        cls.LOGGING_ENDPOINT = cls.HOST.add_path('api/logging')



from typing import Type
import threading

from shortesttrack_tools.functional import classproperty


class Unique:
    """
    This class is implemented the Unique pattern (like Singletone, but without extra meta-programming magic)
    and should be subclassed if you don't need to re-initialize some object on every operation.
    Your subclass's object will be initialized only once, but will have all the power you need.

    >>> class Cat(Unique):
    ...     eyes: int = None
    ...     tail: int = None
    ...     objects_counter: int = None
    ...
    ...     @classmethod
    ...     def _do_init(cls):
    ...         cls.eyes = 2
    ...         cls.tail = 1
    ...         if cls.objects_counter is None:
    ...             cls.objects_counter = 0
    ...         cls.objects_counter += 1
    ...         if cls.objects_counter > 1:
    ...             raise Exception('This exception never be happened')
    ...
    >>> class Dog(Unique):
    ...     eyes: int = None
    ...     tail: int = None
    ...     objects_counter: int = None
    ...
    ...     @classmethod
    ...     def _do_init(cls):
    ...         cls.eyes = 2
    ...         cls.tail = 1
    ...         if cls.objects_counter is None:
    ...             cls.objects_counter = 0
    ...         cls.objects_counter += 1
    ...         if cls.objects_counter > 1:
    ...             raise Exception('This exception never be happened')
    ...
    >>> cat = Cat.init()
    >>> Cat._initialized != Dog._initialized
    True
    >>> cat.eyes
    2
    >>> cat.objects_counter
    1
    >>> cat.eyes +=1
    >>> cat.eyes
    3
    >>> another_but_same_cat = Cat.init()
    >>> another_but_same_cat.eyes
    3
    >>> another_but_same_cat.objects_counter
    1
    >>> another_but_same_cat == cat
    True
    >>> id(another_but_same_cat) == id(cat)
    True
    >>> dog = Dog.init()
    >>> dog.eyes
    2
    >>> cat.eyes
    3
    >>> cat3 = Cat.init()
    >>> cat3.eyes
    3
    >>> dog.tail
    1
    >>> dog == cat
    False
    >>> dog.objects_counter
    1
    >>> dog._init_lock != cat._init_lock
    True
    """

    _initialized = False
    _init_lock = None

    def __init__(self):
        raise RuntimeError('The Unique subclass cannot be initialized in this way. '
                           'Do not use __init__ for initialize the Unique object.')

    @classmethod
    def init(cls, *args, **kwargs) -> 'Type[Unique]':
        """
        Use to call this method instead of __init__ (like you do for your regular classes)

        Returns
        -------
        object
            cls itself
        """

        if cls._initialized:
            return cls

        if cls._init_lock:
            init_lock = cls._init_lock
        else:
            init_lock = cls._init_lock = threading.RLock()

        with init_lock:
            if cls._initialized:
                return cls

            cls._do_init(*args, **kwargs)
            cls._finish_init()
            return cls

    @classmethod
    def _do_init(cls, *args, **kwargs):
        """
        This method is the regular classes __init__ method analogue.

        Define your initialized logic in this method of your subclass.

        Notes
        -----
        It will do effect only once, while the first initialization.
        """
        raise NotImplementedError

    @classmethod
    def _finish_init(cls):
        cls._initialized = True

    @classproperty
    def initialized(cls):
        return cls._initialized



from logging import StreamHandler

from shortesttrack_tools.logger.clients.base_logger_client import BaseLoggerClient


class STLoggerHandler(StreamHandler):
    def __init__(self, client: BaseLoggerClient):
        from shortesttrack_tools.logger import getLogger
        self._client = client
        self._meta_logger = getLogger('st-logger-handler.meta-logger')
        super().__init__()

    def emit(self, record):
        message = self.format(record)
        try:
            self._client.send(message)
        except Exception:
            self._meta_logger.exception('Error sending message')



import abc
from datetime import datetime

import six
from pytz import UTC

from .public_key_mixin import PublicKeyMixin
from .. import jwt_util


@six.add_metaclass(abc.ABCMeta)
class SecAccessTokenMixin(PublicKeyMixin):
    @abc.abstractmethod
    def _get_cache_sec_access_token(self, configuration_id):
        """
        Return sec access token from cache

        """

    @abc.abstractmethod
    def _set_cache_sec_access_token(self, configuration_id, access_token):
        """
        Set sec refresh token to cache

        """

    @abc.abstractmethod
    def _get_cache_sec_access_token_expiration(self, configuration_id):
        """
        Return sec access token expiration time (as Python datetime object)

        """

    @abc.abstractmethod
    def _set_cache_sec_access_token_expiration(self, configuration_id, expiration):
        """
        Return sec access token expiration time (as Python datetime object)

        """

    @property
    @abc.abstractmethod
    def _basic_auth_tuple(self):
        """
        Return basic auth tuple with name and password

        """

    @abc.abstractmethod
    def get_sec_refresh_token(self, configuration_id):
        """
        Return refresh token

        """

    def _reset_cache_public_key(self):
        # TODO: make method abstract for required overriding
        pass

    def get_sec_access_token(self, configuration_id):
        if not self._is_sec_access_token_expired(configuration_id):
            # Get cached access token from store
            return self._get_cache_sec_access_token(configuration_id)

        refresh_token = self.get_sec_refresh_token(configuration_id)

        data = 'grant_type=refresh_token&client_id=script&refresh_token={}'.format(refresh_token)

        json = self.request(self.POST, 'token',
                            base=self.endpoint_urls.OAUTH_SERVICE_ENDPOINT,
                            data=data,
                            extra_headers={
                                'Content-Type': 'application/x-www-form-urlencoded',
                                'Accept': 'application/json'
                            },
                            basic_auth_tuple=self._basic_auth_tuple)

        access_token = json['access_token']
        expiration_datetime = self.parse_access_token_expiration(access_token)
        self._set_cache_sec_access_token(configuration_id, access_token)
        self._set_cache_sec_access_token_expiration(configuration_id, expiration_datetime)

        return access_token

    def _is_sec_access_token_expired(self, configuration_id):
        sec_access_token_expiration = self._get_cache_sec_access_token_expiration(configuration_id)

        if not sec_access_token_expiration:
            return True

        now = self._get_time_now()
        seconds_diff = (sec_access_token_expiration - now).total_seconds()

        if seconds_diff <= 20:
            return True

        return False

    def _get_time_now(self):
        return datetime.now(UTC)

    def parse_access_token_expiration(self, access_token):
        public_key = self.get_public_key()
        util = jwt_util.JWTUtil(access_token)
        exp_string = util.extract_expiration_string(public_key)
        expiration_datetime = datetime.utcfromtimestamp(exp_string).replace(tzinfo=UTC)
        return expiration_datetime



import logging
import sys
from typing import Iterable

from shortesttrack_tools.logger.channels import LogChannel
from shortesttrack_tools.logger.clients.prototype_client import PrototypeLoggerClient
from shortesttrack_tools.logger.handlers.st_handler import STLoggerHandler
from shortesttrack_tools.logger.registry.logger_registry import LoggerRegistry

APPLICATION_LOG_NAME = 'shortesttrack-tools'
PROTOTYPE_LOG_NAME = 'prototype-logger'

DEFAULT_FORMATTER = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
PROTOTYPE_FORMATTER = logging.Formatter('%(name)s - %(levelname)s - %(message)s')

LOG_LEVEL = 'INFO'
root_logger = logging.getLogger()
root_logger.setLevel(LOG_LEVEL)

stream_handler = logging.StreamHandler(sys.stdout)
stream_handler.setFormatter(DEFAULT_FORMATTER)
root_logger.addHandler(stream_handler)


def getLogger(name: str = APPLICATION_LOG_NAME, log_level: str = LOG_LEVEL,
              extra_handlers: Iterable[logging.Handler] = None):
    logger = logging.getLogger(name)

    registry = LoggerRegistry.init()
    if id(logger) in registry.logger_ids:
        # Prevent to duplicate handlers and re-installing options
        return logger

    logger.setLevel(log_level)

    if extra_handlers:
        for handler in extra_handlers:
            logger.addHandler(handler)

    registry.logger_ids.add(id(logger))
    return logger


def _get_qualified_name(name: str) -> str:
    if name:
        return f'{APPLICATION_LOG_NAME}.{name}'

    return APPLICATION_LOG_NAME


def get_prototype_logger(
        name: str = PROTOTYPE_LOG_NAME,
        channel: LogChannel = LogChannel.PERFORMANCE.value,
        prototype_formatter=PROTOTYPE_FORMATTER,
        log_level=LOG_LEVEL
):
    client = PrototypeLoggerClient(channel)
    handler = STLoggerHandler(client)
    handler.setFormatter(prototype_formatter)
    _logger = getLogger(name, log_level, [handler])
    return _logger



from shortesttrack_tools.api_client.endpoints.base_endpoint import BaseEndpoint


class MetadataEndpoint(BaseEndpoint):
    def __init__(self, api_client, script_execution_configuration_id, *args, **kwargs):
        super().__init__(api_client, base=api_client.endpoint_urls.METADATA_SERVICE_ENDPOINT,
                         script_execution_configuration_id=script_execution_configuration_id, *args, **kwargs)



from setuptools import setup, find_packages

setup(
    name='shortesttrack-tools',
    version='0.1.7',
    packages=find_packages(),
    install_requires=[
        'URLObject==2.4.3',
        'requests==2.19.1',
        'requests-oauthlib==1.0.0',
        'pytz==2018.5',
        'PyJWT==1.6.1',
        'cryptography==2.2.1'
    ]
)
